@@ -47,7 +47,7 @@ use UNIVERSAL;
 
 use vars qw( $VERSION @ISA %Attrib );
 
-$VERSION = '1.08';
+$VERSION = '1.09';
 @ISA     = qw( Class::Attrib DynaLoader );
 %Attrib  = (
 	Mode		=> 0660,
@@ -84,8 +84,12 @@ sub bind($$) {
 	my ( $self );
 
 	unless ( $self = $this->attach( $ipckey ) ) {
-		$self = $this->create( $ipckey, $size, $mode );
+
+		$self = $this->create( $ipckey, $size, $mode )
+			or return undef;
+
 		$self->unlock;
+
 	}
 
 	return $self;
@@ -126,7 +130,7 @@ sub attach($$) {
 	if ( my $shmid = $ShmIndex{$ipckey} ) {
 		my $share = $ShmShare{$shmid};
 		unless ( $share ) {
-			confess( __PACKAGE__ . "->attach: dangling ShmIndex" );
+			carp __PACKAGE__ . "->attach: dangling ShmIndex";
 			delete $ShmIndex{$ipckey};
 			return undef;
 		}
@@ -140,6 +144,8 @@ sub attach($$) {
 			return $self;
 		}
 
+		carp __PACKAGE__ . "->attach: got invalid cached object";
+
 		delete $ShmCount{$shmid};
 		delete $ShmShare{$shmid};
 		delete $ShmIndex{$ipckey};
@@ -147,11 +153,18 @@ sub attach($$) {
 		return undef;
 	}
 
-	my $share = sharelite_attach( $ipckey )
-		or return undef; # FIXME should we squawk?
+	my $share = sharelite_attach( $ipckey );
+
+	return undef unless $share; # no carp, $! is set
 
 	my $shmid = sharelite_shmid( $share );
 
+	unless ( defined $shmid ) {
+		carp "sharelite_shmid returned undef";
+		sharelite_shmdt( $share );
+		return undef;
+	}
+
 	bless my $self = {}, $class;
 	$self->{share} = $share;
 	$self->{shmid} = $shmid;
@@ -207,15 +220,21 @@ sub create($;$) {
 	$size   ||= $this->Size();
 	$mode   ||= $this->Mode();
 
-	# FIXME sanity checks?
-
 	my $class = ref( $this ) || $this;
 
-	my $share = sharelite_create( $ipckey, $size, $mode )
-		or return undef; # FIXME should we squawk?
+	my $share = sharelite_create( $ipckey, $size, $mode );
+
+	return undef unless $share; # no carp, $! is set
 
 	my $shmid = sharelite_shmid( $share );
 
+	unless ( defined $shmid ) {
+		carp "sharelite_shmid returned undef";
+		sharelite_remove( $share );
+		sharelite_shmdt( $share );
+		return undef;
+	}
+
 	bless my $self = {}, $class;
 	$self->{share} = $share;
 	$self->{shmid} = $shmid;
@@ -239,12 +258,6 @@ sub shmat($$) {
 	confess( __PACKAGE__ . "->shmat: Called without shmid." )
 		unless defined $shmid;
 
-	confess( __PACKAGE__ . "->shmat: Called with empty shmid." )
-		unless $shmid;
-
-	confess( __PACKAGE__ . "->shmat: Called with string shmid." )
-		unless $shmid != 0;
-
 	confess( __PACKAGE__ . "->shmat: Called with invalid shmid." )
 		if $shmid == -1;
 
@@ -261,14 +274,17 @@ sub shmat($$) {
 			return $self;
 		}
 
+		carp __PACKAGE__ . "->shmat: got invalid cached object";
+
 		delete $ShmCount{$shmid};
 		delete $ShmShare{$shmid};
 
 		return undef;
 	}
 
-	my $share = sharelite_shmat( $shmid )
-		or return undef;
+	my $share = sharelite_shmat( $shmid );
+
+	return undef unless $share; # no carp, $! is set
 
 	bless my $self = {}, $class;
 	$self->{share} = $share;
@@ -300,8 +316,12 @@ sub remove($) {
 	my ( $self ) = @_;
 	my ( $share, $shmid, $ipckey );
 
-	$share  = $self->{share}
-		or return undef; # FIXME: squawk?
+	$share  = $self->{share};
+
+	unless ( $share ) {
+		carp "undefined share during remove";
+		return undef;
+	}
 
 	return ( sharelite_remove( $share ) == -1 ) ? undef : 1;
 }
@@ -311,8 +331,12 @@ sub remove($) {
 sub DESTROY($) {
 	my ( $self ) = @_;
 
-	my $shmid = $self->{shmid}
-		or return; # FIXME squawk
+	my $shmid = $self->{shmid};
+
+	unless ( defined $shmid ) {
+		carp "undefined shmid during DESTROY";
+		return;
+	}
 
 	$ShmCount{$shmid}--;
 
@@ -340,11 +364,19 @@ sub DETACH {
 
 	$self->scache_clean;
 
-	my $shmid = $self->{shmid}
-		or return; # FIXME squawk
+	my $shmid = $self->{shmid};
 
-	my $share = $self->{share}
-		or return; # FIXME squawk
+	unless ( defined $shmid ) {
+		carp "undefined shmid during DETACH";
+		return;
+	}
+
+	my $share = $self->{share};
+
+	unless ( $share ) {
+		carp "undefined share during DETACH";
+		return;
+	}
 
 	my $ipckey = sharelite_key( $share );
 
@@ -500,7 +532,12 @@ my %ShmCache = ();		# cache key=shmid  value={}
 sub scache($) {
 	my $self = shift;
 
-	my $shmid = $self->{shmid}; # FIXME or squawk
+	my $shmid = $self->{shmid};
+
+	unless ( defined $shmid ) {
+		carp "undefined shmid during scache retrieval";
+		return undef;
+	}
 
 	my $cache = $ShmCache{$shmid} ||= {};
 
@@ -523,7 +560,19 @@ sub fetch($) {
 		if $self->_locked( LOCK_UN );
 
 	my $share = $self->{share};
+
+	unless ( $share ) {
+		carp "undefined share during fetch";
+		return undef;
+	}
+
 	my $shmid = $self->{shmid};
+
+	unless ( defined $shmid ) {
+		carp "undefined shmid during fetch";
+		return undef;
+	}
+
 	my $cache = $ShmCache{$shmid} ||= {};
 
 	# determine current shared memory value serial number
@@ -576,7 +625,20 @@ sub store($$) {
 		unless $self->_locked( LOCK_EX );
 
 	my $share = $self->{share};
-	my $cache = $ShmCache{$self->{shmid}} ||= {};
+
+	unless ( $share ) {
+		carp "undefined share during store";
+		return undef;
+	}
+
+	my $shmid = $self->{shmid};
+
+	unless ( $shmid ) {
+		carp "undefined shmid during store";
+		return undef;
+	}
+
+	my $cache = $ShmCache{$shmid} ||= {};
 
 	my $rc = sharelite_store( $share, $_[0], CORE::length( $_[0] ) );
 
@@ -617,10 +679,17 @@ sub lock($$) {
 sub _lock($$) {
 	my ( $self, $flag ) = @_;
 
-	# short circuit if already locked as requested
-	return 0 if sharelite_locked( $self->{share}, $flag );
+	my $share = $self->{share};
 
-	my $rc = sharelite_lock( $self->{share}, $flag );
+	unless ( $share ) {
+		carp "undefined share during _lock";
+		return undef;
+	}
+
+	# short circuit if already locked as requested
+	return 0 if sharelite_locked( $share, $flag );
+
+	my $rc = sharelite_lock( $share, $flag );
 
 	if ( $rc == -1 ) {
 		carp( __PACKAGE__ . "->_lock: $!" );
@@ -637,7 +706,14 @@ sub locked($$) {
 sub _locked($$) {
 	my ( $self, $flag ) = @_;
 
-	my $rc = sharelite_locked( $self->{share}, $flag );
+	my $share = $self->{share};
+
+	unless ( $share ) {
+		carp "undefined share during _locked";
+		return undef;
+	}
+
+	my $rc = sharelite_locked( $share, $flag );
 
 	if ( $rc == -1 ) {
 		carp( __PACKAGE__ . "->_locked: $!" );
